﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace SVIctl_CSharp
{
    /// <summary>
    /// メイン画面
    /// </summary>
    public partial class FormSVMCtl : Form
    {
        #region ▼ Field
        // ボード情報格納配列
        private UInt32 boardNum;
        private UInt32[] boardInfo;
        private UInt32 selectedBoardInfo;

        /// <summary>
        /// Setting File Write Dialogの初期設定ファイル名
        /// </summary>
        private string settingFileName;

        #endregion

        #region ▼ Property
        #endregion

        #region ▼ Constructor
        /// <summary>
        /// コンストラクタ
        /// </summary>
        public FormSVMCtl()
        {
            // デザイナコンポーネントを初期化
            this.InitializeComponent();

            comboBoxSelectBoard.DropDownStyle = ComboBoxStyle.DropDownList;

        }
        #endregion

        #region ▼ Event
        /// <summary>
        /// フォームが読み込まれるときの処理。
        /// </summary>
        /// <param name="sender">発生元</param>
        /// <param name="e">添付情報</param>
        private void FormSVMCtl_Load(object sender, EventArgs e)
        {

            // 画像入力ライブラリ開始処理
            SVIControl.ApiInit();

            // 接続ボード枚数取得
            this.boardNum = SVIControl.GetBoardNum();

            // 接続ボードがない場合
            if (this.boardNum == 0)
            {
                // 一部のボタンを無効化
                this.buttonSettingFileWrite.Enabled = false;
                this.buttonWrite.Enabled = false;
                this.buttonRead.Enabled = false;
            }
            else
            {
                // 接続ボードのBoard番号格納配列宣言
                this.boardInfo = new UInt32[this.boardNum];

                // 接続ボードのBoard番号を取得
                this.boardInfo = SVIControl.GetBoardInfo();

                // 接続ボードのBoard番号をコンボボックスのアイテムに追加
                foreach (UInt32 i in boardInfo)
                {
                    this.comboBoxSelectBoard.Items.Add((i >> 16).ToString("X4"));
                }

                // コンボボックスItem Index0を選択
                this.comboBoxSelectBoard.SelectedIndex = 0;
            }

            // 保存された設定値を読み込む
            this.LoadSVMCtlSetting();

            // ReadValueテキストボックスの折返しを無効化
            this.textBoxReadValue.WordWrap = false;
        }

        /// <summary>
        /// フォームが閉じられるときの処理
        /// </summary>
        /// <param name="sender">発生元</param>
        /// <param name="e">添付情報</param>
        private void FormSVMCtl_FormClosing(object sender, FormClosingEventArgs e)
        {
            // 設定値を保存する
            this.SaveSVMCtlSetting();

            // ライブラリ終了処理
            SVIControl.DeviceClose();

            // 画像入力ライブラリ終了処理
            SVIControl.ApiEnd();
        }


        private void comboBoxSelectBoard_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (SVIControl.deviceOpenFlag == true)
            {
                // ライブラリ終了処理
                SVIControl.DeviceClose();
            }

            // 選択されたボードのBoard番号を代入
            selectedBoardInfo = this.boardInfo[this.comboBoxSelectBoard.SelectedIndex];

            // ライブラリ初期化とコンボボックスで選択されたBoard番号のボードを選択
            SVIControl.DeviceOpen(selectedBoardInfo);
        }



        private void buttonScanSVI03_Click(object sender, EventArgs e)
        {
            // コンボボックスをクリア
            this.comboBoxSelectBoard.Text = "";
            this.comboBoxSelectBoard.Items.Clear();

            if (SVIControl.deviceOpenFlag == true)
            {
                // ライブラリ終了処理
                SVIControl.DeviceClose();
            }

            // 接続ボード枚数取得
            this.boardNum = SVIControl.GetBoardNum();

            // 接続ボードがない場合
            if (this.boardNum == 0)
            {
                // 一部のボタンを無効化
                this.buttonSettingFileWrite.Enabled = false;
                this.buttonWrite.Enabled = false;
                this.buttonRead.Enabled = false;
            }
            else
            {
                // 一部のボタンを有効化
                this.buttonSettingFileWrite.Enabled = true;
                this.buttonWrite.Enabled = true;
                this.buttonRead.Enabled = true;

                // 接続ボードのBoard番号格納配列初期化
                this.boardInfo = null;
                this.boardInfo = new UInt32[this.boardNum];

                // 接続ボードのBoard番号を取得
                this.boardInfo = SVIControl.GetBoardInfo();

                // 接続ボードのBoard番号をコンボボックスのアイテムに追加
                foreach (UInt32 i in boardInfo)
                {
                    this.comboBoxSelectBoard.Items.Add((i >> 16).ToString("X4"));
                }

                this.comboBoxSelectBoard.SelectedIndex = 0;
            }
        }


        /// <summary>
        /// SettingFileWriteボタンのクリックイベント。
        /// </summary>
        /// <param name="sender">発生元</param>
        /// <param name="e">添付情報</param>
        private void buttonSettingFileWrite_Click(object sender, EventArgs e)
        {
            // 設定ファイル情報代入変数
            UInt32 sfwSlaveAddress = 0;
            UInt32 sfwSubAddress;
            byte[] sfwDataByteArray;
            bool sfwWordAddressMode;
            int sfwUnit = 16;

            // WordAddressMode初期値設定
            sfwWordAddressMode = this.checkBoxWordAddressMode.Checked;

            // ファイルを開くダイアログ
            OpenFileDialog openFileDialog = new OpenFileDialog();
            // ファイル名
            openFileDialog.FileName = this.settingFileName;
            // 「ファイルの種類」の選択肢
            openFileDialog.Filter = "I2C通信用設定ファイル(*.txt)|*.txt";
            // ダイアログのタイトル
            openFileDialog.Title = "I2C通信用設定ファイルを指定してください。";

            // ファイルを開くダイアログを表示し、キャンセルされた場合
            if (DialogResult.OK != openFileDialog.ShowDialog())
            {
                // 以降の処理はしない
                return;
            }

            // 開いた設定ファイル名をiniファイルの参照先に代入
            this.settingFileName = openFileDialog.FileName;

            // ボタン無効化
            this.buttonSettingFileWrite.Enabled = false;
            this.buttonRead.Enabled = false;
            this.buttonWrite.Enabled = false;

            // ファイル読み込み
            using (StreamReader reader = new StreamReader(openFileDialog.FileName, Encoding.GetEncoding("Shift_JIS")))
            {
                string line;
                int lineCount = 0;
                int waitValue = 0;


                // １行ずつ読み込む
                while ((line = reader.ReadLine()) != null)
                {
                    lineCount++;

                    // 左右空白削除, 大文字変換
                    line = line.Trim().ToUpper();

                    // 空白行スキップ
                    if (String.IsNullOrWhiteSpace(line))
                    {
                        continue;
                    }

                    // 「;」で始まる場合（1行全てコメント）
                    if (line.StartsWith(";") || line.StartsWith("#"))
                    {
                        continue;
                    }

                    // 行間に含まれるコメントを削除
                    if (line.Contains(";"))
                    {
                        line = line.Substring(0, line.IndexOf(";"));
                    }
                    if (line.Contains("#"))
                    {
                        line = line.Substring(0, line.IndexOf("#"));
                    }

                    // wait処理
                    if (line.StartsWith("WT"))
                    {
                        if (int.TryParse(line.Replace("WT", "").Replace(",", "").Trim(), out waitValue))
                        {
                            Thread.Sleep(waitValue);
                            continue;
                        }
                        else
                        {
                            break;
                        }
                    }

                    // 命令詞BYTE行の不要文字列チェック
                    if (line == "BYTE")
                    {
                        sfwWordAddressMode = false;
                        continue;
                    }

                    // 命令詞WORD行の不要文字列チェック
                    if (line == "WORD")
                    {
                        sfwWordAddressMode = true;
                        continue;
                    }

                    // 命令詞UNIT行の不要文字列チェック
                    if (line.StartsWith("UNIT"))
                    {
                        // 基数抽出, エラーチェック, sfwUnitへの代入
                        if (((int.TryParse(line.Replace("UNIT", "").Replace(",", "").Trim(), out sfwUnit) == true) && (sfwUnit == 10 || sfwUnit == 16)) == true)
                        {
                            continue;
                        }
                    }

                    // 命令詞SLAVE行の不要文字列チェック
                    if (line.StartsWith("SLAVE"))
                    {
                        // エラーチェックとsfwSlaveAddressへの代入
                        if (this.IsHex(line.Replace("SLAVE", "").Replace(",", "").Trim()))
                        {
                            sfwSlaveAddress = Convert.ToUInt32(line.Replace("SLAVE", "").Replace(",", "").Trim(), sfwUnit);
                            continue;
                        }
                    }

                    // I2C Write実行
                    if (line.Contains(","))
                    {
                        // UInt32に変換可能な場合、変換後代入
                        if (UInt32.TryParse(line.Substring(0, line.IndexOf(",")), out sfwSubAddress))
                        {
                            sfwSubAddress = Convert.ToUInt32(line.Substring(0, line.IndexOf(",")), sfwUnit);
                        }


                        sfwDataByteArray = StringToByteArray(line.Substring(line.IndexOf(",") + 1, line.Length - (line.IndexOf(",") + 1)), sfwUnit);

                        // I2C書き込み
                        if (SVIControl.SVI_I2C_OneBlockWrite(sfwSlaveAddress, sfwSubAddress, sfwDataByteArray, sfwWordAddressMode) == NativeMethods_SVI.RET_NORMAL)
                        {
                            continue;
                        }
                        else
                        {
                            // エラーメッセージBOX
                            MessageBox.Show("Error:\n" + lineCount + ": " + line, "Settings File Write Error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                            MessageBox.Show("カメラレジスタの読込に失敗しました。\nボードとの接続や設定内容を確認してください。", "SVMctl", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                            break;
                        }

                    }

                    // エラーメッセージBOX
                    MessageBox.Show("Error:\n" + lineCount + ": " + line, "Settings File Write Error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    break;
                }
            }

            // デバイス終了処理
            SVIControl.DeviceClose();

            // ボタン有効化
            this.buttonSettingFileWrite.Enabled = true;
            this.buttonRead.Enabled = true;
            this.buttonWrite.Enabled = true;
        }

        /// <summary>
        /// Writeボタンのクリックイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonWrite_Click(object sender, EventArgs e)
        {
            uint i2cSlaveAddress;
            uint i2cSubAddress;
            byte[] writeValueByteArray;

            #region 入力値チェック
            // Slave Address
            string i2cSlaveAddressText = this.textBoxI2CSlaveAddress.Text.Trim();

            // I2CSlaveAddress欄が空欄の場合
            if (string.IsNullOrWhiteSpace(i2cSlaveAddressText) == true)
            {
                MessageBox.Show("I2C Slave Addressが未設定です。設定してください。", "SVMctl", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                // I2CSlaveAddress欄に移動
                this.textBoxI2CSlaveAddress.Focus();

                // 以降の処理はしない
                return;
            }

            // I2CSlaveAddress欄の入力値が16進数以外の場合
            if (this.IsHex(i2cSlaveAddressText) == false)
            {
                MessageBox.Show("Slave Addressに16進数以外が設定されてます。再設定してください。", "SVMctl", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                // I2CSlaveAddress欄に移動
                this.textBoxI2CSlaveAddress.Focus();

                // 以降の処理はしない
                return;
            }
            else
            {
                i2cSlaveAddress = Convert.ToUInt32(i2cSlaveAddressText, 16);
            }


            // Sub Address
            string i2cSubAddressText = this.textBoxSubAddress.Text.Trim();

            // I2CSubAddress欄が空欄の場合
            if (string.IsNullOrWhiteSpace(i2cSubAddressText) == true)
            {
                MessageBox.Show("Sub Addressが未設定です。設定してください。", "SVMctl", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                // I2CSlaveAddress欄に移動
                this.textBoxSubAddress.Focus();

                // 以降の処理はしない
                return;
            }

            // SubAddress欄の入力値が16進数以外の場合
            if (this.IsHex(i2cSubAddressText) == false)
            {
                MessageBox.Show("Sub Addressに16進数以外が設定されてます。再設定してください。", "SVMctl", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                // SubAddress欄に移動
                this.textBoxSubAddress.Focus();

                // 以降の処理はしない
                return;
            }
            else
            {
                i2cSubAddress = Convert.ToUInt32(i2cSubAddressText, 16);
            }


            // Write Value確認

            // WriteValue欄が空欄の場合
            if (string.IsNullOrWhiteSpace(this.textBoxWriteValue.Text) == true)
            {
                MessageBox.Show("Write Valueが未設定です。設定してください。", "SVMctl", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                // WriteValue欄に移動
                this.textBoxWriteValue.Focus();

                // 以降の処理はしない
                return;
            }

            // WriteValue欄の入力値が16進数以外の場合
            if (this.IsHex(this.textBoxWriteValue.Text.Replace(",", "")) == false)
            {
                MessageBox.Show("Write Valueに16進数以外が設定されてます。再設定してください。", "SVMctl", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                // SubAddress欄に移動
                this.textBoxWriteValue.Focus();

                // 以降の処理はしない
                return;
            }



            #endregion 入力値チェック

            writeValueByteArray = this.StringToByteArray(this.textBoxWriteValue.Text);

            // StringToByteArray関数の戻り値がnullの場合 (Write Valueの値が1byteを超える場合)
            if (writeValueByteArray == null)
            {
                MessageBox.Show("Write Valueの値が不正です。確認してください。", "SVMctl", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                // WriteValue欄に移動
                this.textBoxWriteValue.Focus();

                // 以降の処理はしない
                return;
            }

            // I2C通信を実行
            // データの書き込みに失敗した場合
            if (SVIControl.SVI_I2C_OneBlockWrite(i2cSlaveAddress, i2cSubAddress, writeValueByteArray, this.checkBoxWordAddressMode.Checked, this.checkBoxRestartCondition.Checked) == null)
            {
                // I2C通信に失敗(通信APIの戻り値が異常値)の場合
                MessageBox.Show("カメラレジスタへの設定に失敗しました。\n設定ファイルの内容、カメラやボードとの接続を確認してください。",
                    "SVMctl", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
            }
        }

        /// <summary>
        /// Readボタンのクリックイベント
        /// </summary>
        /// <param name="sender">発生元</param>
        /// <param name="e">添付情報</param>
        private void buttonRead_Click(object sender, EventArgs e)
        {
            uint i2cSlaveAddress;
            uint i2cSubAddress;
            uint readCounts;

            #region 入力値チェック

            // Slave Address
            string i2cSlaveAddressText = this.textBoxI2CSlaveAddress.Text.Trim();

            // I2CSlaveAddress欄が空欄の場合
            if (string.IsNullOrWhiteSpace(i2cSlaveAddressText) == true)
            {
                MessageBox.Show("I2C Slave Addressが未設定です。設定してください。", "SVMctl", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                // I2CSlaveAddress欄に移動
                this.textBoxI2CSlaveAddress.Focus();

                // 以降の処理はしない
                return;
            }

            // I2CSlaveAddress欄の入力値が16進数以外の場合
            if (this.IsHex(i2cSlaveAddressText) == false)
            {
                MessageBox.Show("I2C Slave Addressに16進数以外が設定されてます。再設定してください。", "SVMctl", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                // I2CSlaveAddress欄に移動
                this.textBoxI2CSlaveAddress.Focus();

                // 以降の処理はしない
                return;
            }
            else
            {
                i2cSlaveAddress = Convert.ToUInt32(i2cSlaveAddressText, 16);
            }


            // Sub Address
            string i2cSubAddressText = this.textBoxSubAddress.Text.Trim();

            // I2CSubAddress欄が空欄の場合
            if (string.IsNullOrWhiteSpace(i2cSubAddressText) == true)
            {
                MessageBox.Show("Sub Addressが未設定です。設定してください。", "SVMctl", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                // I2CSlaveAddress欄に移動
                this.textBoxSubAddress.Focus();

                // 以降の処理はしない
                return;
            }

            // SubAddress欄の入力値が16進数以外の場合
            if (this.IsHex(i2cSubAddressText) == false)
            {
                MessageBox.Show("Sub Addressに16進数以外が設定されてます。再設定してください。", "SVMctl", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                // SubAddress欄に移動
                this.textBoxSubAddress.Focus();

                // 以降の処理はしない
                return;
            }
            else
            {
                i2cSubAddress = Convert.ToUInt32(i2cSubAddressText, 16);
            }


            // Read Counts
            // ReadCounts欄に英数字以外が入力されている場合
            if (uint.TryParse(this.textBoxReadCounts.Text.Trim(), out readCounts) == false)
            {
                MessageBox.Show("Read Countsに数値以外が設定されてます。再設定してください。", "SVMctl", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                // ReadCounts欄に移動
                this.textBoxReadCounts.Focus();

                // 以降の処理はしない
                return;
            }
            #endregion


            // 表示用文字列を定義
            StringBuilder readValue = new StringBuilder();

            // 通信実行
            // I2Cデータ読み出しを実行
            byte[] dataArray = SVIControl.SVI_I2C_OneBlockRead(i2cSlaveAddress, i2cSubAddress, readCounts, this.checkBoxWordAddressMode.Checked, this.checkBoxRestartCondition.Checked);
            
            // データの読み出しに失敗した場合
            if (dataArray == null || dataArray.Length == 0)
            {
                MessageBox.Show("カメラレジスタの読込に失敗しました。\n設定ファイルの内容、カメラやボードとの接続を確認してください。", "SVMctl", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);

                // 以降の処理はしない
                return;
            }

            // 表示用文字列整形
            for (int i = 0; i < readCounts; i++)
            {
                // 16進数2桁
                readValue.Append(dataArray[i].ToString("X2"));
                readValue.Append(",");
            }


            // 末尾のカンマを削除
            readValue.Length--;
            readValue.AppendLine();

            // ReadValue欄に追記
            this.textBoxReadValue.Text += readValue.ToString();

            // ReadValue欄内最終行に移動
            this.textBoxReadValue.SelectionStart = this.textBoxReadValue.Text.Length;
            this.textBoxReadValue.ScrollToCaret();
        }

        /// <summary>
        /// Clearボタンのクリックイベント
        /// </summary>
        /// <param name="sender">発生元</param>
        /// <param name="e">添付情報</param>
        private void buttonClear_Click(object sender, EventArgs e)
        {
            this.textBoxReadValue.Text = "";
        }

        /// <summary>
        /// テキストボックスがアクティブになったときの共通処理
        /// </summary>
        /// <param name="sender">発生元</param>
        /// <param name="e">添付情報</param>
        private void TextBox_Enter(object sender, EventArgs e)
        {
            // テキストを全選択
            ((TextBox)sender).SelectAll();
        }
        #endregion

        #region ▼ Private Method
        /// <summary>
        /// iniファイルに保存された設定値を読み込み
        /// </summary>
        private void LoadSVMCtlSetting()
        {
            // iniファイルに保存された設定値を画面へ反映
            this.textBoxI2CSlaveAddress.Text = IniFileController.GetString(IniFileController.Keys.SlaveAddress);
            this.textBoxSubAddress.Text = IniFileController.GetString(IniFileController.Keys.SubAddress);
            this.textBoxWriteValue.Text = IniFileController.GetString(IniFileController.Keys.WriteValue);
            this.textBoxReadCounts.Text = IniFileController.GetString(IniFileController.Keys.ReadCounts);
            this.checkBoxWordAddressMode.Checked = IniFileController.GetBool(IniFileController.Keys.WordAddressMode);
            this.settingFileName = IniFileController.GetString(IniFileController.Keys.SettingFileName);

        }

        /// <summary>
        /// iniファイルへ設定値を保存
        /// </summary>
        private void SaveSVMCtlSetting()
        {
            IniFileController.WriteString(IniFileController.Keys.SlaveAddress, this.textBoxI2CSlaveAddress.Text);
            IniFileController.WriteString(IniFileController.Keys.SubAddress, this.textBoxSubAddress.Text);
            IniFileController.WriteString(IniFileController.Keys.WriteValue, this.textBoxWriteValue.Text);
            IniFileController.WriteString(IniFileController.Keys.ReadCounts, this.textBoxReadCounts.Text);
            IniFileController.WriteBool(IniFileController.Keys.WordAddressMode, this.checkBoxWordAddressMode.Checked);
            IniFileController.WriteString(IniFileController.Keys.SettingFileName, this.settingFileName);
        }

        /// <summary>
        /// 文字列が16進数に変換可能かどうかを検証
        /// </summary>
        /// <param name="target">検証対象の文字列</param>
        /// <returns>true..16進数に変換可能、false..16進数に変換不可</returns>
        private bool IsHex(string target)
        {
            target = target.Trim().ToLower().Replace("0x", "");

            // 空欄なら変換不可
            if (string.IsNullOrWhiteSpace(target) == true)
            {
                return false;
            }

            // 16進数文字でない文字が含まれるなら変換不可
            foreach (char c in target)
            {
                if (Uri.IsHexDigit(c) == false)
                {
                    return false;
                }
            }

            return true;
        }


        /// <summary>
        /// string型文字列をByte型配列に変換
        /// </summary>
        /// <param name="stringData">変換元string型文字列</param>
        /// <returns>正常時:変換後のByte型配列 異常時:null</returns>
        private byte[] StringToByteArray(string stringData, int conversionUnit = 16)
        {
            // 区切り文字
            char[] delimiter = { ',', ' ' };

            // 対象文字列が空の場合
            if (string.IsNullOrWhiteSpace(stringData) == true)
            {
                return null;
            }

            // 前後空白除去
            stringData = stringData.Trim();

            // 対象文字列を分割して取得(空白要素は除去)
            string[] stringDataArray = stringData.Split(delimiter, StringSplitOptions.RemoveEmptyEntries);

            byte[] byteArray = new byte[stringDataArray.Length];

            for (int i = 0; i < stringDataArray.Length; i++)
            {
                UInt32 temp;

                // Write Valueに"0x FF"等の不正な値が入力された場合の例外処理
                if (this.IsHex(stringDataArray[i]) == true)
                {
                    temp = Convert.ToUInt32(stringDataArray[i], conversionUnit);
                }
                else
                {
                    return null;
                }

                // 1byte超でnullを返す
                if (temp > 0xFF)
                {
                    return null;
                }

                byteArray[i] = (byte)temp;

            }
            return byteArray;
        }


        #endregion

        #region ▼ Public Method
        #endregion

        #region ▼ Override Method
        #endregion


    }
}
